﻿/******************************************************************************
Send bug reports to: philip@spigelmire.com

Copyright (c) 2010, Weatherite Corporation
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE 
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
******************************************************************************/

using System;
using System.Text;

namespace Opto22Library
{
    internal enum TransactionCode
    {
        SIOMM_TCODE_WRITE_QUAD_REQUEST = 0,
        SIOMM_TCODE_WRITE_BLOCK_REQUEST = 1,
        SIOMM_TCODE_WRITE_RESPONSE = 2,
        SIOMM_TCODE_READ_QUAD_REQUEST = 4,
        SIOMM_TCODE_READ_BLOCK_REQUEST = 5,
        SIOMM_TCODE_READ_QUAD_RESPONSE = 6,
        SIOMM_TCODE_READ_BLOCK_RESPONSE = 7
    }

    public struct SIOMM_DigPointReadArea
    {
        public bool State;          // bool
        public bool OnLatch;        // bool
        public bool OffLatch;       // bool
        public bool CounterState;   // bool
        public UInt32 Counts;         // unsigned int
    }

    public struct SIOMM_DigBankReadArea
    {
        public UInt64 StatePoints;
        public UInt64 OnLatchStatePoints;
        public UInt64 OffLatchStatePoints;
        public UInt64 ActiveCountersPoints;
    }

    public struct SIOMM_AnaPointReadArea
    {
        public Single Value;
        public Single Counts;
        public Single MinValue;
        public Single MaxValue;
    }

    public struct SIOMM_PointConfigArea
    {
        public UInt32 ModuleType;       // Read only.  Not used in SetPointConfiguration()
        public UInt32 PointType;        // Read/Write
        public UInt32 Feature;          // Read/Write.  Only used for digital points
        public Single Offset;           // Read/Write.  Only used for analog points
        public Single Gain;             // Read/Write.  Only used for analog points
        public Single HiScale;          // Read/Write.  Only used for analog points
        public Single LoScale;          // Read/Write.  Only used for analog points
        public Single WatchdogValue;    // Read/Write
        public bool WatchdogEnabled;  // Read/Write
    }

    public struct SIOMM_PointConfigArea2
    {
        public UInt32 ModuleType;          // Read only.  Not used in SetPointConfiguration()
        public UInt32 PointType;           // Read/Write
        public UInt32 Feature;             // Read/Write.  Only used for digital points
        public Single Offset;              // Read/Write.  Only used for analog points
        public Single Gain;                // Read/Write.  Only used for analog points
        public Single HiScale;             // Read/Write.  Only used for analog points
        public Single LoScale;             // Read/Write.  Only used for analog points
        public Single FilterWeight;        // Read/Write.  Only used for analog points
        public Single WatchdogValue;       // Read/Write
        public bool WatchdogEnabled;     // Read/Write
        private string name; //Reference type - don't like this, but strings are immutable so acceptable for now.

        public string sName {
            get { return name; }
            set { name = value.Substring(0, SIOMM.MM.POINT.CONFIG_NAME_SIZE < value.Length ? (int)SIOMM.MM.POINT.CONFIG_NAME_SIZE : value.Length); } 
        }
        public byte[] bName {
            get {
                byte[] rv = new byte[SIOMM.MM.POINT.CONFIG_NAME_SIZE];

                System.Text.ASCIIEncoding ae = new ASCIIEncoding();
                ae.GetBytes(name).CopyTo(rv, 0);
                return rv;
            }
             
            set {
                System.Text.ASCIIEncoding ae = new ASCIIEncoding();
                name = ae.GetString(value).Substring(0, SIOMM.MM.POINT.CONFIG_NAME_SIZE < value.Length ? (int)SIOMM.MM.POINT.CONFIG_NAME_SIZE : value.Length);
            } 
        }

        //public fixed char Name[16]; //Read/Write  //This would require unsafe modifier
    }

    public struct SIOMM_PointConfigArea2X
    {
        public bool Valid;
        public UInt32 ModuleType;          // Read only.  Not used in SetPointConfiguration()
        public UInt32 PointType;           // Read/Write
        public UInt32 Feature;             // Read/Write.  Only used for digital points
        public Single Offset;              // Read/Write.  Only used for analog points
        public Single Gain;                // Read/Write.  Only used for analog points
        public Single HiScale;             // Read/Write.  Only used for analog points
        public Single LoScale;             // Read/Write.  Only used for analog points
        public Single FilterWeight;        // Read/Write.  Only used for analog points
        public Single WatchdogValue;       // Read/Write
        public bool WatchdogEnabled;     // Read/Write
        public Single LoClamp;
        public Single HiClamp;
        public String Name; //Reference type - don't like this, but strings are immutable so acceptable for now.
    }

    public struct SIOMM_StatusVersion
    {
        public UInt32 MapVer;            //  Memory map version
        public SIOMM_Version LoaderVersion;     //  Loader version (1.2.3.4 format)
        public SIOMM_Version KernelVersion;     //  Kernel version (1.2.3.4 format)
    }

    public struct SIOMM_Version
    {
        public byte Major;
        public byte Minor;
        public byte Revision;
        public byte Letter;
        public override string ToString()
        {
            return String.Format("{0}.{1}.{2}.{3}", Major, Minor, Revision, (char)Letter);
        }
    }

    public struct SIOMM_StatusHardware
    {
        public UInt32 IoUnitType;        //  I/O unit type
        public byte HwdVerMonth;      //  hardware version (month)
        public byte HwdVerDay;        //  hardware version (day)
        public UInt16 HwdVerYear;        //  hardware version (4 digit year)
        public UInt32 RamSize;           //  bytes of installed RAM
    }

    public struct SIOMM_StatusHardware2
    {
        public UInt32 IoUnitType;        //  I/O unit type
        public byte HwdVerMonth;      //  hardware version (month)
        public byte HwdVerDay;        //  hardware version (day)
        public UInt16 HwdVerYear;        //  hardware version (4 digit year)
        public UInt32 RamSize;           //  bytes of installed RAM
        private string partNumber; //Reference type - don't like this, but strings are immutable so acceptable for now.

        public string sPartNumber
        {
            get { return partNumber; }
            set { partNumber = value.Substring(0, SIOMM.MM.STATUS.READ_PART_NUMBER_SIZE < value.Length ? (int)SIOMM.MM.STATUS.READ_PART_NUMBER_SIZE : value.Length); }
        }
        public byte[] bPartNumber
        {
            get
            {
                byte[] rv = new byte[SIOMM.MM.STATUS.READ_PART_NUMBER_SIZE];

                System.Text.ASCIIEncoding ae = new ASCIIEncoding();
                ae.GetBytes(partNumber).CopyTo(rv, 0);
                return rv;
            }

            set
            {
                System.Text.ASCIIEncoding ae = new ASCIIEncoding();
                partNumber = ae.GetString(value).Substring(0, SIOMM.MM.STATUS.READ_PART_NUMBER_SIZE < value.Length ? (int)SIOMM.MM.STATUS.READ_PART_NUMBER_SIZE : value.Length);
            }
        }
    }

    public struct SIOMM_StatusNetwork
    {
        public UInt64 MACAddress;
        public SIOMM_IPAddress TCPIPAddress;      //  IP address
        public SIOMM_IPAddress SubnetMask;        //  subnet mask
        public SIOMM_IPAddress DefGateway;        //  default gateway
    }

    public struct SIOMM_StatusNetwork2
    {
        public UInt64 MACAddress;
        public SIOMM_IPAddress TCPIPAddress;      //  IP address
        public SIOMM_IPAddress SubnetMask;        //  subnet mask
        public SIOMM_IPAddress DefGateway;        //  default gateway
        public UInt32 TcpIpMinRtoMS;     //  TCP/IP minimum Response Timeout (RTO) in milliseconds
        public UInt32 InitialRtoMS;      //  initial RTO
        public UInt32 TcpRetries;        //  number of TCP retries
        public UInt32 TcpIdleTimeout;    //  TCP idle session timeout
        public UInt32 EnetLateCol;       //  Ethernet late collisions
        public UInt32 EnetExcessiveCol;  //  Ethernet excessive collisions
        public UInt32 EnetOtherErrors;   //  Other Ethernet errors
    }

    public struct SIOMM_IPAddress
    {
        public SIOMM_IPAddress(string IPAddress)
        {
            string [] splitString = IPAddress.Split('.');

            Octet1 = Convert.ToByte(splitString[0]);
            Octet2 = Convert.ToByte(splitString[1]);
            Octet3 = Convert.ToByte(splitString[2]);
            Octet4 = Convert.ToByte(splitString[3]);
        }

        public byte Octet1;
        public byte Octet2;
        public byte Octet3;
        public byte Octet4;

        public override string ToString()
        {
            return String.Format("{0}.{1}.{2}.{3}", Octet1, Octet2, Octet3, Octet4);
        }
    }

    public struct SIOMM_StreamStandardBlock
    {
        public UInt16 DataLength; //16 bits
        public byte Tag; //2 bits
        public byte Channel; //6 bits
        public byte TransactionCode; //4 bits
        public byte SyncCode; //4 bits;

        // The following items map directly to the standard stream data
        public Single[] AnalogValue; //Initialize to 64 values
        public UInt32[] DigPointFeature;  //Initialize to 64 values
        public UInt64 State;
        public UInt64 OnLatchState;
        public UInt64 OffLatchState;
        public UInt64 ActiveCounters;

        public SIOMM_IPAddress TCPIPAddress;
    }

    public struct SIOMM_SerialModuleConfigArea
    {
        public UInt32 IpPort;            //  READ ONLY
        public UInt32 BaudRate;          //  baud rate
        public byte Parity;           //  parity
        public byte DataBits;         //  data bits (7 or 8)
        public byte StopBits;         //  stop bits (1 or 2)
        public bool HardwareFlowControl;
        public bool SendTestMessage;      //  bool for sending a powerup test message
        public byte EOM1;             //  first  end-of-message character
        public byte EOM2;             //  second end-of-message character
        public byte EOM3;             //  third  end-of-message character
        public byte EOM4;             //  fourth end-of-message character
    }

    public struct SIOMM_StreamConfiguration
    {
        public bool OnFlag;
        public UInt32 IntervalMS;
        public UInt32 Port;
        public bool IoMirroringEnabled;
        public UInt32 StartAddress;
        public UInt32 DataSize;
    }


    public enum ConnectionType
    {
        UDP, TCP
    }

    public enum TemperatureUnits
    {
        Fahrenheit, Celcius
    }

    public enum SIOMM_OpCode
    {
        PowerUpClear=1,
        ResetToDefault=2,
        StoreConfigToFlash=3,
        EraseConfigFromFlash=4,
        ResetHardware=5,
        ClearDigitalEvents=6,
        ClearAlarmConfig=7,
        ClearPPPConfig=8,
        ClearEmailConfig=9,
        ClearDigitalEventsExpanded=10,
        ClearPIDConfig=11,
        ClearDataLog=12//,
        //BootToLoader=0x87654321
    }

    public static class SIOMM
    {
        //Response codes
        public const int OK = 1;

        public const int ERROR_BAD_TCODE = -1;  //Used
        //public const int TIME_OUT = -2; //Socket exception handles this - unneeded.
        //public const int ERROR_NO_SOCKETS = -3;  // Unable to access socket interface
        //public const int ERROR_CREATING_SOCKET = -4;
        //public const int ERROR_CONNECTING_SOCKET = -5;
        public const int ERROR_RESPONSE_BAD = -6;  //Used
        //public const int ERROR_NOT_CONNECTED_YET = -7;
        //public const int ERROR_OUT_OF_MEMORY = -8;
        public const int ERROR_NOT_CONNECTED = -9; //Used
        //public const int ERROR_STREAM_TYPE_BAD = -10;

        //Internal
        public const int ERROR_PAYLOAD_TOO_BIG = -100;

        // Error values generated by the I/O brain itself
        public const int BRAIN_ERROR_UNDEFINED_CMD = 0xE001;
        public const int BRAIN_ERROR_INVALID_PT_TYPE = 0xE002;
        public const int BRAIN_ERROR_INVALID_FLOAT = 0xE003;
        public const int BRAIN_ERROR_PUC_EXPECTED = 0xE004;
        public const int BRAIN_ERROR_INVALID_ADDRESS = 0xE005;
        public const int BRAIN_ERROR_INVALID_CMD_LENGTH = 0xE006;
        public const int BRAIN_ERROR_RESERVED = 0xE007;
        public const int BRAIN_ERROR_BUSY = 0xE008;
        public const int BRAIN_ERROR_CANT_ERASE_FLASH = 0xE009;
        public const int BRAIN_ERROR_CANT_PROG_FLASH = 0xE00A;
        public const int BRAIN_ERROR_IMAGE_TOO_SMALL = 0xE00B;
        public const int BRAIN_ERROR_IMAGE_CRC_MISMATCH = 0xE00C;
        public const int BRAIN_ERROR_IMAGE_LEN_MISMATCH = 0xE00D;
        public const int BRAIN_ERROR_FEATURE_NOT_IMPL = 0xE00E;
        public const int BRAIN_ERROR_WATCHDOG_TIMEOUT = 0xE00F;


        // Sizes of the read/write requests and responses
        public const uint SIZE_WRITE_QUAD_REQUEST = 16;
        public const uint SIZE_WRITE_BLOCK_REQUEST = 16;
        public const uint SIZE_WRITE_RESPONSE = 12;
        public const uint SIZE_READ_QUAD_REQUEST = 12;
        public const uint SIZE_READ_QUAD_RESPONSE = 16;
        public const uint SIZE_READ_BLOCK_REQUEST = 16;
        public const uint SIZE_READ_BLOCK_RESPONSE = 16;

        // Response codes from the I/O unit
        public const byte RESPONSE_CODE_ACK = 0;
        public const byte RESPONSE_CODE_NAK = 7;

        // Memory Map values
        public static class MM
        {

            public static class STATUS
            {

                // Status read area of the memory map
                public const uint READ_BASE = 0xF0300000;
                public const uint READ_PUC_FLAG = 0xF0300004;
                public const uint READ_LAST_ERROR = 0xF030000C;
                public const uint READ_TRANS_LABEL = 0xF0300010;
                public const uint READ_SOURCE_ADDRESS = 0xF0300012;
                public const uint READ_ERROR_ADDRESS = 0xF0300014;
                public const uint READ_LOADER_REVISION = 0xF0300018;
                public const uint READ_FIRMWARE_REVISION = 0xF030001C;
                public const uint READ_UNIT_TYPE = 0xF0300020;
                public const uint READ_BOOTP_FLAG = 0xF0300048;
                public const uint READ_DEGREES_FLAG = 0xF030004C;
                public const uint READ_WATCHDOG_TIME = 0xF0300054;
                public const uint READ_PART_NUMBER = 0xF0300080;
                public const uint READ_PART_NUMBER_SIZE = 0x00000020;
                // Status write area of the memory map
                public const uint WRITE_OPERATION = 0xF0380000;
                public const uint WRITE_BOOTP = 0xF0380004;
                public const uint WRITE_TEMP_DEGREES = 0xF0380008;
                public const uint WRITE_WATCHDOG_TIME = 0xF0380010;
            }

            public static class DATE_AND_TIME
            {

                // Date and Time
                public const uint BASE = 0xF0350000;
                public const uint SIZE = 0x00000017;
            }

            public static class NETWORK_SECURITY
            {
                public const uint MMP_PORT = 0xF03A0004;

            }
            public static class SERIAL_MODULE
            {
                // Serial Module
                public const uint CONFIG_BASE = 0xF03A8000;
                public const uint CONFIG_BOUNDARY = 0x00000010;
                public const uint EOM_BASE = 0xF03A8200;
                public const uint EOM_BOUNDARY = 0x00000010;
            }

            public static class STREAM
            {
                // Streaming configuration area of the memory map
                public const uint CONFIG_BASE = 0xF03FFFC4;
                public const uint CONFIG_ON_FLAG = 0xF03FFFD0;
                public const uint CONFIG_INTERVAL = 0xF03FFFD4;
                public const uint CONFIG_PORT = 0xF03FFFD8;

                public const uint TARGET_BOUNDARY = 0x00000004;
                public const uint TARGET_BASE = 0xF03FFFE0;

                // Streaming read area of the memory map
                public const uint READ_AREA_BASE = 0xF1000000;
                public const uint READ_AREA_SIZE = 0x00000220;
            }

            public static class DBANK
            {
                // Digital bank read area of the memory map
                public const uint READ_AREA_BASE = 0xF0400000;
                public const uint READ_POINT_STATES = 0xF0400000;
                public const uint READ_ON_LATCH_STATES = 0xF0400008;
                public const uint READ_OFF_LATCH_STATES = 0xF0400010;
                public const uint READ_ACTIVE_COUNTERS = 0xF0400018;
                public const uint READ_COUNTER_DATA_BASE = 0xF0400100;
                public const uint READ_COUNTER_DATA_BOUNDARY = 0x00000004;
                public const uint MAX_FEATURE_ELEMENTS = 0x00000040;

                // Digital bank write area of the memory map
                public const uint WRITE_AREA_BASE = 0xF0500000;
                public const uint WRITE_TURN_ON_MASK = 0xF0500000;
                public const uint WRITE_TURN_OFF_MASK = 0xF0500008;
                public const uint WRITE_ACT_COUNTERS_MASK = 0xF0500010;
                public const uint WRITE_DEACT_COUNTERS_MASK = 0xF0500018;

            }
            public static class ABANK
            {
                // Analog bank read area of the memory map
                public const uint READ_AREA_BASE = 0xF0600000;
                public const uint READ_POINT_VALUES = 0xF0600000;
                public const uint READ_POINT_COUNTS = 0xF0600100;
                public const uint READ_POINT_MIN_VALUES = 0xF0600200;
                public const uint READ_POINT_MAX_VALUES = 0xF0600300;

                // Analog bank write area of the memory map
                public const uint WRITE_AREA_BASE = 0xF0700000;
                public const uint WRITE_POINT_VALUES = 0xF0700000;
                public const uint WRITE_POINT_COUNTS = 0xF0700100;

                public const uint MAX_BYTES = 0x00000100;
                public const uint MAX_ELEMENTS = 0x00000040;

            }

            public static class POINT
            {
                public static class DPOINT
                {
                    // Digital point read area of the memory map
                    public const uint READ_BOUNDARY = 0x00000040;
                    public const uint READ_AREA_BASE = 0xF0800000;
                    public const uint READ_STATE = 0xF0800000;
                    public const uint READ_ONLATCH_STATE = 0xF0800004;
                    public const uint READ_OFFLATCH_STATE = 0xF0800008;
                    public const uint READ_ACTIVE_COUNTER = 0xF080000C;
                    public const uint READ_COUNTER_DATA = 0xF0800010;

                    // Digital point write area of the memory map
                    public const uint WRITE_BOUNDARY = 0x00000040;
                    public const uint WRITE_TURN_ON_BASE = 0xF0900000;
                    public const uint WRITE_TURN_OFF_BASE = 0xF0900004;
                    public const uint WRITE_ACTIVATE_COUNTER = 0xF0900008;
                    public const uint WRITE_DEACTIVATE_COUNTER = 0xF090000C;

                    // Digital point read and clear area of the memory map
                    public const uint READ_CLEAR_BOUNDARY = 0x00000004;
                    public const uint READ_CLEAR_BOUNDARY_X = 0x00000018;
                    public const uint READ_CLEAR_COUNTS_BASE = 0xF0F00000;
                    public const uint READ_CLEAR_COUNTS_BASE_X = 0xF02E0000;
                    public const uint READ_CLEAR_ON_LATCH_BASE = 0xF0F00100;
                    public const uint READ_CLEAR_ON_LATCH_BASE_X = 0xF02E0004;
                    public const uint READ_CLEAR_OFF_LATCH_BASE = 0xF0F00200;
                    public const uint READ_CLEAR_OFF_LATCH_BASE_X = 0xF02E0008;

                    //High density digital - added 2020 04 23
                    public const uint READ_HDD_STATE = 0xF1808000;
                    public const uint READ_HDD_ON_LATCH_STATE = 0xF1808008;
                    public const uint READ_HDD_OFF_LATCH_STATE = 0xF1808010;
                    public const uint READ_HDD_STATE_BOUNDARY = 0x00000040;
                    public const uint WRITE_HDD_STATE_ON = 0xF180C000;
                    public const uint WRITE_HDD_STATE_OFF = 0xF180C008;
                    public const uint WRITE_HDD_STATE_BOUNDARY = 0x00000040;
                    public const uint READ_HDD_COUNTER_DATA = 0xF1809000;
                    public const uint READ_HDD_COUNTER_MODULE_BOUNDARY = 0x00000100;
                    public const uint READ_HDD_COUNTER_CHANNEL_BOUNDARY = 0x00000004;

                }

                public static class APOINT
                {
                    // Analog point read area of the memory map
                    public const uint READ_BOUNDARY = 0x00000040;
                    public const uint READ_BOUNDARY_X = 0x00000040;
                    public const uint READ_AREA_BASE = 0xF0A00000;
                    public const uint READ_AREA_BASE_X = 0xF0260000; 
                    public const uint READ_VALUE_BASE = 0xF0A00000;
                    public const uint READ_VALUE_BASE_X = 0xF0260000;
                    public const uint READ_COUNTS_BASE = 0xF0A00004;
                    public const uint READ_COUNTS_BASE_X = 0xF0260004;
                    public const uint READ_MIN_VALUE_BASE = 0xF0A00008;
                    public const uint READ_MIN_VALUE_BASE_X = 0xF0260008;
                    public const uint READ_MAX_VALUE_BASE = 0xF0A0000C;
                    public const uint READ_MAX_VALUE_BASE_X = 0xF026000C;
                    public const uint READ_TPO_PERIOD_BASE = 0xF0B0000C;
                    public const uint READ_TPO_PERIOD_BASE_X = 0xF02A000C;

                    // Analog point write area of the memory map
                    public const uint WRITE_BOUNDARY = 0x00000040;
                    public const uint WRITE_BOUNDARY_X = 0x00000040;
                    public const uint WRITE_VALUE_BASE = 0xF0B00000;
                    public const uint WRITE_VALUE_BASE_X = 0xF02A0000;
                    public const uint WRITE_COUNTS_BASE = 0xF0B00004;
                    public const uint WRITE_COUNTS_BASE_X = 0xF02A0004;
                    public const uint WRITE_TPO_PERIOD_BASE = 0xF0B0000C;
                    public const uint WRITE_TPO_PERIOD_BASE_X = 0xF02A000C;


                    // Analog point read and clear area of the memory map
                    public const uint READ_CLEAR_BOUNDARY = 0x00000004;
                    public const uint READ_CLEAR_BOUNDARY_X = 0x0000000C;
                    public const uint READ_CLEAR_MIN_VALUE_BASE = 0xF0F80000;
                    public const uint READ_CLEAR_MIN_VALUE_BASE_X = 0xF01D4000;
                    public const uint READ_CLEAR_MAX_VALUE_BASE = 0xF0F80100;
                    public const uint READ_CLEAR_MAX_VALUE_BASE_X = 0xF01D4004;

                    // Analog point calc and set area of the memory map
                    public const uint READ_CALC_SET_BOUNDARY = 0x00000004;
                    public const uint READ_CALC_SET_BOUNDARY_X = 0x00000008;
                    public const uint READ_CALC_SET_OFFSET_BASE = 0xF0E00000;
                    public const uint READ_CALC_SET_OFFSET_BASE_X = 0xF01C0000;
                    public const uint READ_CALC_SET_GAIN_BASE = 0xF0E00100;
                    public const uint READ_CALC_SET_GAIN_BASE_X = 0xF01C0004;
                }


                // Point configuration area of the memory map
                public const uint CONFIG_BOUNDARY = 0x00000040;
                public const uint CONFIG_BOUNDARY_X = 0x000000C0;
                public const uint CONFIG_READ_MOD_TYPE_BASE = 0xF0C00000;
                public const uint CONFIG_READ_MOD_TYPE_BASE_X = 0xF0100000;
                public const uint CONFIG_WRITE_TYPE_BASE = 0xF0C00004;
                public const uint CONFIG_WRITE_TYPE_BASE_X = 0xF0100004;
                public const uint CONFIG_WRITE_FEATURE_BASE = 0xF0C00008;
                public const uint CONFIG_WRITE_FEATURE_BASE_X = 0xF0100008;
                public const uint CONFIG_WRITE_OFFSET_BASE = 0xF0C0000C;
                public const uint CONFIG_WRITE_OFFSET_BASE_X = 0xF010000C;
                public const uint CONFIG_WRITE_GAIN_BASE = 0xF0C00010;
                public const uint CONFIG_WRITE_GAIN_BASE_X = 0xF0100010;
                public const uint CONFIG_WRITE_HISCALE_BASE = 0xF0C00014;
                public const uint CONFIG_WRITE_HISCALE_BASE_X = 0xF0100014;
                public const uint CONFIG_WRITE_LOSCALE_BASE = 0xF0C00018;
                public const uint CONFIG_WRITE_LOSCALE_BASE_X = 0xF0100018;
                public const uint CONFIG_WRITE_WDOG_VALUE_BASE = 0xF0C00024;
                public const uint CONFIG_WRITE_WDOG_VALUE_BASE_X = 0xF0100024;
                public const uint CONFIG_WRITE_WDOG_ENABLE_BASE = 0xF0C00028;
                public const uint CONFIG_WRITE_WDOG_ENABLE_BASE_X = 0xF0100028;

                public const uint CONFIG_NAME_SIZE = 0x00000010;
                public const uint CONFIG_NAME_SIZE_X = 0x00000033;
            }

            public static class SCRATCHPAD
            {

                // Scratch pad aread of the memory map
                public const uint BITS_BASE = 0xF0D80000;
                public const uint BITS_ON_MASK_BASE = 0xF0D80400;
                public const uint BITS_OFF_MASK_BASE = 0xF0D80408;
                public const uint INTEGER_BASE = 0xF0D81000;
                public const uint INTEGER_MAX_BYTES = 0x00001000;
                public const uint INTEGER_MAX_ELEMENTS = 0x00000400; //256
                public const uint FLOAT_BASE = 0xF0D82000;
                public const uint FLOAT_MAX_BYTES = 0x00001000;
                public const uint FLOAT_MAX_ELEMENTS = 0x00000400;
                public const uint STRING_BASE = 0xF0D83000;
                public const uint STRING_LENGTH_BASE = 0xF0D83000;
                public const uint STRING_LENGTH_SIZE = 0x00000002;
                public const uint STRING_DATA_BASE = 0xF0D83002;
                public const uint STRING_DATA_SIZE = 0x00000080;
                public const uint STRING_BOUNDARY = 0x00000082;
                public const uint STRING_MAX_BYTES = 0x00002080;
                public const uint STRING_MAX_ELEMENTS = 0x00000040;

                //Firmware 8.1 and higher expanded scratchpad section

                public const uint INTEGER_BASE_X = 0xF0DA0000;
                public const uint INTEGER_MAX_BYTES_X = 0x00009000;
                public const uint INTEGER_MAX_ELEMENTS_X = 0x00002400;
                public const uint FLOAT_BASE_X = 0xF0DC0000;
                public const uint FLOAT_MAX_BYTES_X = 0x00009000;
                public const uint FLOAT_MAX_ELEMENTS_X = 0x00002400;
                //New 64 bit scratchpad section - only on firmware 8.1 and higher
                public const uint INTEGER64_BASE = 0xF0DE0000;
                public const uint INTEGER64_MAX_BYTES = 0x00002000;
                public const uint INTEGER64_MAX_ELEMENTS = 0x00000400;

            }

        }


    }

}
